# coding: utf-8
from fabric.api import env, local, task, roles, run, runs_once, settings
from datetime import datetime
from .web import restart as _restart, restart_back
from .utils import execute, xput

# env.cdn_host = 'cdn.51cxin.cn'


@task
@roles('web')
def upload_etc(app='web'):
    config = '%s/etc/config' % env.path
    xput('etc/%s.py' % app, config)


@task
@roles('media')
@runs_once
def dist_media(source='../media/web/dist', target=None):
    now = datetime.now().strftime('%Y%m%d%H%M')
    target = target or '%s%s' % (env.project, now)

    local('cp -r %s /tmp/%s' % (source, target))
    run('mkdir -p /var/htdocs/www')
    local(r"""

expect -c "
spawn scp -r /tmp/%s %s@%s:/var/htdocs/www
expect {
    \"*assword\" {set timeout 500; send \"%s\r\";}
    \"yes/no\" {send \"yes\r\"; exp_continue;}
}
expect eof"

        """ % (target, env.user, env.host, env.password))


@task
@runs_once
def media(source='../media/web/dist', target=None, app='web', restart=True):
    restart = True if restart in ['True', 'true', True] else False
    now = datetime.now().strftime('%Y%m%d%H%M')
    target = target or '%s%s' % (env.project, now)

    execute(dist_media)

    for stage, e in env.envs.iteritems():
        if stage not in env.exclude_envs:
            prefix = 'SITE_STATIC_PREFIX'
            file = r'%s/etc/%s.py' % (stage, app)
            line = r'%s = \"http://%s/%s/\"' % (prefix, env.cdn_host, target)

            tpl = (
                r'[[ ! `grep %(prefix)s %(file)s` ]] && '
                r'echo "%(line)s" >> %(file)s || '
                r'sed -r -i "s#%(prefix)s.*#%(line)s#" %(file)s'
            )
            print tpl % dict(prefix=prefix, file=file, line=line)
            local(tpl % dict(prefix=prefix, file=file, line=line))

    with settings(stage='all'):
        execute(upload_etc, app)

    if restart:
        with settings(stage='all'):
            execute(_restart)
            execute(restart_back)



# coding: utf-8
from fabric.api import cd, env, settings
from .utils import FabricException, xrun, roles, task


def _start(*args):
    for arg in args:
        with cd(env.path):
            xrun('uwsgi --ini %s/etc/uwsgi/%s.ini' % (env.path, arg),
                 envs='LOGGER_DEBUG=True')


def _start_back(*args):
    for arg in args:
        with cd(env.path):
            xrun('uwsgi --ini %s/etc/uwsgi/%s.ini' % (
                env.path, arg), envs='CHIKI_BACK=true')


def _stop(*args):
    for arg in args:
        with settings(abort_exception=FabricException):
            try:
                xrun('uwsgi --stop %s/run/%s.pid' % (env.path, arg))
            except FabricException as e:
                print str(e)


@roles('web')
@task
def start():
    _start(*env.apps)


@roles('web')
@task
def stop():
    _stop(*env.apps)


@roles('web')
@task
def restart():
    stop()
    start()


@roles('web')
@task
def start_back():
    _start_back(*['%s.back' % x for x in env.apps])


@roles('web')
@task
def stop_back():
    _stop(*['%s.back' % x for x in env.apps])


@roles('web')
@task
def restart_back():
    stop_back()
    start_back()



# coding: utf-8
import os
from fabric.api import cd, env, run, roles, task, lcd
from fabric.api import settings, local, runs_once, put
from fabric.contrib.files import exists, append
from .nginx import nginx
from .server import restart, restart_back
from .utils import execute, xrun, xput, scp


@task
def all():
    execute(init)
    execute(create_env)
    execute(pip)
    execute(deploy)
    execute(nginx)


def build():
    run('apt-get update')
    run(
        'apt-get install -y vim software-properties-common'
        ' python-setuptools nginx --force-yes'
        ' libmysqlclient-dev git gcc g++ unzip'
        ' python-virtualenv python-dev subversion curl'
        ' libxml2-dev libxslt1-dev libfreetype6-dev'
        ' libjpeg62 libpng3 libjpeg-dev libpng12-dev'
        ' libffi-dev libssl-dev expect sshpass'
    )
    run('easy_install pip supervisor')
    run('pip install virtualenvwrapper')

    run('sed -i "s/#   StrictHostKeyChecking ask/'
        'StrictHostKeyChecking no/g" /etc/ssh/ssh_config')


def create_user(name):
    if not exists('/home/%s' % name):
        run('echo "\n\n\n\n\nY\n" | adduser --disabled-password -q %s' % name)
        run('usermod -p $(openssl passwd -1 %s) %s' % (
            env.user_password, name))


@roles('web')
@task
def remove_user(name=None):
    if not name:
        name = env.user
    with settings(user='root', password=env.sudo_password):
        run('killall -u %s' % name)
        run('userdel %s' % name)
        run('rm -rf /home/%s' % name)


@roles('web')
@task
def init():
    user = env.user
    with settings(user='root', password=env.sudo_password):
        build()
        create_user(user)


@roles('web')
@task
def create_env():
    run('mkdir -p /home/%s/.ssh' % env.user)
    rsa = '/home/%s/.ssh/id_rsa_%s' % (env.user, env.project)
    xput('files/id_rsa', rsa)
    run('chmod 600 %s' % rsa)
    append('~/.ssh/config', 'IdentityFile %s' % rsa)

    profile = '~/.bash_profile_%s' % env.project
    xput('files/bash_profile', profile)
    append('~/.bashrc', 'source %s' % profile)

    run('mkdir -p ~/.pip')
    xput('files/pip.conf', '~/.pip/pip.conf')
    if not exists('/home/%s/.virtualenvs/%s' % (env.user, env.project)):
        cmd = 'source /usr/local/bin/virtualenvwrapper.sh && mkvirtualenv %s'
        run(cmd % env.project)

    for path in [env.path, env.src, env.dist]:
        run('mkdir -p %s' % path)

    dirnames = ['data', 'etc/config', 'etc/uwsgi', 'run', 'logs']
    for dirname in dirnames:
        run('mkdir -p %s/%s' % (env.path, dirname))

    config = '%s/etc/config' % env.path
    uwsgi = '%s/etc/uwsgi' % env.path
    xput('etc/base.py', config)
    for app in env.apps:
        xput('etc/%s.py' % app, config)
        xput('uwsgi/%s.ini' % app, uwsgi)
        xput('uwsgi/%s.back.ini' % app, uwsgi)


@roles('web')
@task
def pip(file='../requirements/prod.txt'):
    put(file, os.path.join(env.path, 'requirements.txt'))
    with cd(env.path):
        # xrun('pip install -r setuptools==35.0.2')
        xrun('pip install -r requirements.txt')


@roles('repo')
@task
def clone(name, folder, repo=None, branch=None, copy=False,
          media='media/web/dist data'):
    if exists(folder + '/.git'):
        run('cd %s && git stash && git pull' % folder)
    else:
        run('git clone %s %s' % (repo or env.repo, folder))

    if branch:
        run('cd %s && git checkout %s' % (folder, branch))

    if exists(folder + '/setup.py'):
        with cd(folder):
            run('rm -rf dist `python setup.py --fullname`')
            run('python setup.py sdist')
            run('mv dist/`python setup.py --fullname`'
                '.tar.gz %s.tar.gz' % name)

    if copy:
        with cd(folder):
            run('tar -zcvf %s.media.tar.gz %s' % (name, media))


@roles('repo')
@task
def srepo(folder, items):
    with cd(folder):
        for key, item in items:
            put(key, item)


@roles('web')
@task
def setup(name, folder, copy=False, is_scp=True):
    source = os.path.join(folder, '%s.tar.gz' % name)
    target = os.path.join(env.dist, '%s.tar.gz' % name)
    if is_scp:
        scp(source, target, env.repo_host, getattr(env, 'repo_password', ''))
    else:
        put(source, target)

    with cd(env.dist):
        run('tar -zxvf %s.tar.gz' % name)
        run("mv `gzip -dc %s.tar.gz | tar tvf -| awk '{print $6}' "
            "| awk -F '/' '{print $1}'|uniq` %s" % (name, name))
        with cd(name):
            xrun('python setup.py install')
        if exists(name):
            run('rm -r %s' % name)

    if copy:
        source = os.path.join(folder, '%s.media.tar.gz' % name)
        target = os.path.join(env.dist, '%s.media.tar.gz' % name)
        if is_scp:
            scp(source, target, env.repo_host, getattr(
                env, 'repo_password', ''))
        else:
            put(source, target)
        with cd(env.path):
            run('tar -zxvf %s' % target)


@task
def clone2setup(name, folder, repo=None, branch=None, copy=False,
                media='media/web/dist data'):
    execute(clone, name, folder, repo, branch, copy=copy, media=media)
    execute(setup, name, folder, copy=copy)


@task
def clone4github():
    repos = {
        'chiki': {
            'repo': 'https://github.com/endsh/chiki.git',
            'branch': 'old',
        },
        'simi': 'git@gitlab.com:xiaoku/simi.git',
        'flask-admin': "https://github.com/flask-admin/flask-admin.git",
    }
    for name, repo in repos.iteritems():
        folder = os.path.join(env.src, name)
        if type(repo) == dict:
            clone2setup(name, folder, repo['repo'], repo['branch'])
        else:
            clone2setup(name, folder, repo)


@task
def sdist(copy=False, media='media/web/dist'):
    copy = True if copy in ['True', 'true', True] else False
    with lcd('../'):
        local('python setup.py sdist')
        local('mv dist/`python setup.py --fullname`'
              '.tar.gz dist/%s.tar.gz' % env.project)

        if copy:
            local('tar -zcvf dist/%s.media.tar.gz %s' % (
                env.project, media))
            execute(srepo, env.dist, [
                ('dist/%s.tar.gz' % env.project,
                 '%s.tar.gz' % env.project),
                ('dist/%s.media.tar.gz' % env.project,
                 '%s.media.tar.gz' % env.project),
            ])
        else:
            execute(srepo, env.dist, [
                ('dist/%s.tar.gz' % env.project,
                 '%s.tar.gz' % env.project),
            ])

    execute(setup, env.project, env.dist, copy=copy)


@task
def simi():
    folder = os.environ.get('SIMI_DIR')
    with lcd(folder):
        local('python setup.py sdist')
        local('mv dist/`python setup.py --fullname`.tar.gz dist/simi.tar.gz')

        execute(srepo, env.dist, [
            ('dist/simi.tar.gz', 'simi.tar.gz'),
        ])

    execute(setup, 'simi', env.dist)


@task
def chiki():
    folder = os.environ.get('CHIKI_DIR')
    with lcd(folder):
        local('python setup.py sdist')
        local('mv dist/`python setup.py --fullname`.tar.gz dist/chiki.tar.gz')

        execute(srepo, env.dist, [
            ('dist/chiki.tar.gz', 'chiki.tar.gz'),
        ])

    execute(setup, 'chiki', env.dist)


@task
def update(copy=False, media='media/web/dist data'):
    copy = True if copy in ['True', 'true', True] else False
    clone2setup(env.project, os.path.join(env.src, env.project), copy=copy)


@task
def deploy():
    execute(update)
    execute(clone4github)


@task
@runs_once
def commit(msg='auto commit', project=None):
    folder = '..'
    branch = 'master'
    if project:
        if project == 'simi':
            folder = os.environ.get('SIMI_DIR')
        elif project == 'chiki':
            folder = os.environ.get('CHIKI_DIR')
            branch = 'old'

    with lcd(folder):
        local('git add --all .')
        local('git commit -m "%s"' % msg)
        local('git push -u origin %s' % branch)


@task
def sync(msg='auto commit'):
    commit(msg)
    execute(update)


@task
def up(msg='auto commit'):
    sync(msg)
    execute(restart)


@task
def upx(msg='auto commit'):
    sync(msg)
    execute(restart)
    execute(restart_back)



# coding: utf-8
import os
import fabric
from fabric.api import cd, env, run, local, roles, task
from fabric.api import put, settings, execute as _execute


class FabricException(Exception):
    pass


def xput(source, target, sudo=False):
    if not source.startswith('/'):
        filename = '%s/%s' % (env.stage, source)
        if os.path.isfile(filename):
            source = filename
        else:
            source = 'common/%s' % source
    return put(source, target, use_sudo=sudo)


def xrun(cmd, envs=''):
    return run('%s ~/.virtualenvs/%s/bin/%s' % (envs, env.project, cmd))


def scp(source, target, host, password=None):
    run("sshpass -p %s scp %s:%s %s" % (
        password or env.password, host, source, target))


def execute(task, *args, **kwargs):
    if 'exclude_envs' not in env:
        env.exclude_envs = ['test']

    if task not in ['stage', 's']:
        if env.stage == 'all':
            for stage, e in env.envs.iteritems():
                if stage not in env.exclude_envs:
                    with settings(stage=stage, **e):
                        _execute(task, *args, **kwargs)
            return
        elif env.stage in env.envs:
            with settings(stage=env.stage, **env.envs[env.stage]):
                return _execute(task, *args, **kwargs)
    return _execute(task, *args, **kwargs)


fabric.main.execute = execute
fabric.api.execute = execute


@task(alias='s')
def stage(stage='all'):
    env.stage = stage


@roles('web', 'db', 'front', 'puppet')
@task
def ssh_config():
    with settings(user='root', password=env.sudo_password):
        with cd('/etc/ssh'):
            run('sed -i "s/#ClientAliveInterval 0/ClientAliveInterval 60/g"'
                ' sshd_config')
            run('sed -i "s/#ClientAliveCountMax 3/ClientAliveCountMax 3/g"'
                ' sshd_config')
            run('service ssh reload')


@roles('web')
@task
def tail_log(name='uwsgi.admin', line=100):
    run('tail -n %d %s/logs/%s.log' % (int(line), env.path, name))


@roles('web')
@task
def git_config():
    run('git config --global user.email "tiger@qq.com"')
    run('git config --global user.name "tiger"')


@roles('main')
@task
def crontab():
    filename = os.path.join(env.path, 'etc/cron.conf')
    xput('files/cron.conf', filename)

    run(r'crontab -l | sed "s#no crontab for %(user)s##g"'
        r' | sed ":a;N;s#\n#@@#g;ta"'
        r' | sed "s=## %(branch)s start.*## %(branch)s end==g"'
        r' | sed "s#@@#\n#g" > ~/cron.conf' % dict(
            user=env.user,
            branch=env.branch,
            filename=filename,
        ))

    run('echo "\n## %s start" >> ~/cron.conf' % env.branch)
    run('cat %s >> ~/cron.conf' % filename)
    run('echo "## %s end\n" >> ~/cron.conf' % env.branch)
    run('cat ~/cron.conf')
    run('crontab ~/cron.conf')


@roles('main')
@task
def clear_crontab():
    run('echo "# clear crontab\n" > ~/cron.conf')
    run('crontab ~/cron.conf')


@roles('main')
@task
def service(command):
    with cd(env.path):
        xrun("%s service %s" % (env.project, command))


@roles('main')
@task
def start_service(name):
    with cd(env.path):
        with settings(abort_exception=FabricException):
            try:
                run('kill -s 15 `cat %s/run/%s.pid`' % (env.path, name))
            except FabricException:
                pass
            run('$(nohup ~/.virtualenvs/%(project)s/bin/%(project)s service'
                ' %(name)s >> %(path)s/logs/%(name)s.log 2>&1 &) && sleep 3'
                % dict(
                    project=env.project,
                    name=name,
                    path=env.path,
                ))


@roles('main')
@task
def stop_service(name):
    with cd(env.path):
        with settings(abort_exception=FabricException):
            try:
                run('kill -s 15 `cat %s/run/%s.pid`' % (env.path, name))
            except FabricException:
                pass


@task
def cmd(text):
    run(text)


@task
def ssh(host=None, user=None):
    if not host:
        host = env.roledefs['main'][0]

    if not user:
        user = env.user

    if '@' in host:
        local('ssh %s' % host)
    else:
        local('ssh %s@%s' % (user, host))



# coding: utf-8
from fabric.api import env, run, roles, task, settings
from .utils import xput


@roles('web')
@task
def nginx():
    with settings(user='root', password=env.sudo_password):
        xput('nginx/web.nginx.conf',
             '/etc/nginx/sites-enabled/%s.nginx.conf' % env.branch)
        run('service nginx reload')


@roles('web')
@task
def nginx_upgrade():
    xput('files/upgrade.html', env.path)
    with settings(user='root', password=env.sudo_password):
        xput('nginx/web.upgrade.nginx.conf',
             '/etc/nginx/sites-enabled/%s.nginx.conf' % env.branch)
        run('service nginx reload')


@roles('web')
@task
def nginx_remove(name=None):
    with settings(user='root', password=env.sudo_password):
        name = name or ('%s.nginx.conf' % env.branch)
        run('rm -f /etc/nginx/sites-enabled/%s' % name)


@roles('front')
@task
def nginx_front(update=False):
    update = True if update in ['True', 'true', True] else False
    if update:
        run('apt-get update && apt-get install nginx -y')

    filename = '/etc/nginx/sites-enabled/%s.front.nginx.conf' % env.branch
    xput('nginx/front.nginx.conf', filename)
    run('service nginx reload')


# @roles('front')
# @task
# def nginx_upgrade(update=True):
#     if update:
#         run('apt-get update && apt-get install nginx -y')

#     filename = '/etc/nginx/sites-enabled/%s.nginx.conf' % env.branch
#     xput('nginx/upgrade.nginx.conf', filename)
#     run('service nginx reload')


@roles('puppet')
@task
def nginx_puppet(update=False):
    update = True if update in ['True', 'true', True] else False
    if update:
        run('apt-get update && apt-get install nginx -y')

    xput('nginx/puppet.nginx.conf',
         '/etc/nginx/sites-enabled/%s.nginx.conf' % env.branch)
    run('service nginx reload')


@roles('puppet')
@task
def nginx_puppet_remove(name=None):
    name = name or ('%s.nginx.conf' % env.branch)
    run('rm -f /etc/nginx/sites-enabled/%s' % name)


@roles('qing')
@task
def nginx_qing(update=False):
    update = True if update in ['True', 'true', True] else False
    if update:
        run('sudo apt-get update && sudo apt-get install nginx -y')

    xput('nginx/qing.nginx.conf',
         '/etc/nginx/sites-enabled/%s.nginx.conf' % env.branch, sudo=True)
    run('sudo service nginx reload')



# coding: utf-8
import re
import os
from setuptools import setup, find_packages


def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)


def read(fname):
    return open(fpath(fname)).read()


file_text = read(fpath('chiki_deploy/__init__.py'))


def grep(attrname):
    pattern = r"{0}\W*=\W*'([^']+)'".format(attrname)
    strval, = re.findall(pattern, file_text)
    return strval


def get_data_files(*dirs):
    results = []
    for src_dir in dirs:
        for root, dirs, files in os.walk(src_dir):
            results.append((root, map(lambda f: root + "/" + f, files)))
    return results


setup(
    name='chiki-deploy',
    version='0.0.1',
    url='https://github.com/endsh/chiki-deploy',
    author='Linshao',
    author_email='438985635@qq.com',
    description='chiki deploy libs.',
    py_modules=['chiki_deploy'],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Fabric==1.10.2',
    ],
)
